﻿
BINDING_HEADER_VOICEHELPER = "VoiceHelper";
BINDING_NAME_VOICEHELPER_CHANNELFRAME = GetText("CHAT_CHANNELS");
BINDING_NAME_VOICEHELPER_VOICESETTINGS = GetText("SETTINGS").."("..GetText("VOICE_CHAT")..")";

local addon = CreateFrame("Button", "VoiceHelperButton", UIParent, "MainMenuBarMicroButton");
addon:RegisterEvent("PLAYER_ENTERING_WORLD");
addon:RegisterEvent("INITIAL_CLUBS_LOADED");
addon:RegisterEvent("VOICE_CHAT_CHANNEL_MEMBER_ACTIVE_STATE_CHANGED");
addon:RegisterEvent("VOICE_CHAT_DEAFENED_CHANGED");
addon:RegisterEvent("VOICE_CHAT_MUTED_CHANGED");
addon:SetClampedToScreen(true);
addon:SetMovable(true);
addon:RegisterForDrag("MiddleButton");
addon:SetScript("OnEvent", function(self, event, ...)
	if event == "PLAYER_ENTERING_WORLD" then
		self:initialize();
	elseif event == "INITIAL_CLUBS_LOADED" then
		if VoiceHelper["auto4"] and not C_VoiceChat.GetActiveChannelID() and C_Club.GetClubInfo(VoiceHelper["auto4"].clubId)then
			local clubId = VoiceHelper["auto4"].clubId;
			local streamId = VoiceHelper["auto4"].streamId;
			ChannelFrame:TryJoinCommunityStreamChannel(clubId, streamId);
			print(GetText("AUTO_JOIN_VOICE").." : "..C_Club.GetClubInfo(clubId).name.." - "..C_Club.GetStreamInfo(clubId, streamId).name);
		end
	elseif event == "VOICE_CHAT_DEAFENED_CHANGED" then
		self:Update();
	elseif event == "VOICE_CHAT_MUTED_CHANGED" then
		self:Update();
	elseif event == "VOICE_CHAT_CHANNEL_MEMBER_ACTIVE_STATE_CHANGED" then
		self:Update();
	end
end);

local function pos()
	addon:ClearAllPoints();
	addon:SetPoint("BOTTOM", ChatFrameChannelButton, "BOTTOM", 0, 0);
end

local onload = false;
function addon:initialize()
	if onload then
		return;
	end
	onload = true;

	if VoiceHelper == nil then
		VoiceHelper = {};
		VoiceHelper["auto4"] = false;
	else
		if VoiceHelper["auto4"] == nil		then VoiceHelper["auto4"] = false; end
	end

	local p, rf, rp, x, y = addon:GetPoint();
	if p == nil then
		pos();
	end
	
	addon:Show();
	addon:SetFrameLevel(4);

	LoadMicroButtonTextures(addon, "Character");
	local headset = addon:CreateTexture();
	local atlasOn = "voicechat-channellist-icon-headphone-on";
	local atlasOff = "voicechat-channellist-icon-headphone-off";
	headset:SetSize(18,18);
	headset:ClearAllPoints();
	headset:SetPoint("TOP", addon, "TOP", 0, -6);
	headset:SetDrawLayer("OVERLAY", 0)
	addon.headset = headset;

	addon:SetScript("OnEnter", function()

	end)

	addon:SetScript("OnLeave", function()

	end)
	
	local pass = false;
	addon:SetScript("OnClick", function(self, btn)
		if pass then
			pass = false;
			return;
		end
		if btn == "LeftButton" then
			ChannelFrame:Toggle();
		elseif btn == "RightButton" then
			ShowOptionsPanel(VideoOptionsFrame, nil, VOICE_CHAT);
		end
	end)

	addon:SetScript("OnDoubleClick", function(self, btn)
		if IsShiftKeyDown() then
			self:SetUserPlaced(false);
			pos();
		end
	end)

	addon:SetScript("OnMouseDown", function(self)
		self.headset:ClearAllPoints()
		self.headset:SetPoint("TOP", addon, "TOP", -1, -8);
		if IsShiftKeyDown() then
			addon:StartMoving();
			pass = true;
		end
	end)

	addon:SetScript("OnMouseUp", function()
		self.headset:ClearAllPoints()
		self.headset:SetPoint("TOP", addon, "TOP", 0, -6);
		addon:StopMovingOrSizing();
	end)

	local activeChannelID = C_VoiceChat.GetActiveChannelID();
	local count = addon:CreateFontString("VoiceHelperButtonCount", "OVERLAY", "GameFontHighlightSmall");
	count:ClearAllPoints();
	count:SetDrawLayer("OVERLAY", 1);
	count:SetPoint("BOTTOM", addon, "BOTTOM", 0, 6);
	addon.count = count;

	self:Update();
end

local function countActiveMembers(vChannelID)
	if not vChannelID then
		return false;
	end
	local channel = C_VoiceChat.GetChannel(vChannelID);
	local numActive = 0;
	for i, v in pairs(channel.members) do
		if v.isActive then
			numActive = numActive + 1;
		end
	end
	return numActive;
end

function addon:Update()
	if not onload then
		return;
	end
	local channelId = C_VoiceChat.GetActiveChannelID();
	if channelId then
		if C_VoiceChat.IsDeafened() then
			self.headset:SetAtlas("voicechat-icon-speaker-mute");
		elseif C_VoiceChat.IsMuted() then
			self.headset:SetAtlas("voicechat-icon-mic-mute");
		else
			self.count:SetText(countActiveMembers(channelId));
			self.headset:SetAtlas("voicechat-channellist-icon-headphone-on");
		end
		local r, g, b = Voice_GetVoiceChannelNotificationColor(channelId);
		addon.Flash:SetVertexColor(r, g, b, 0.5);
		addon.Flash:Show();
	else
		self.count:SetText("");
		self.headset:SetAtlas("voicechat-channellist-icon-headphone-off");
		addon.Flash:Hide();
	end
end

local function AllCheck()
	local buttons = {ChannelFrame.ChannelList.Child:GetChildren()};
	for i, button in pairs(buttons) do
		if button.name then
			if button:ChannelSupportsVoice() and button:GetChannelType() == 4 then
				if VoiceHelper["auto4"] and VoiceHelper["auto4"].clubId == button.clubId and VoiceHelper["auto4"].streamId == button.streamId then
					button.auto.AutoCastShine:Show();
					AutoCastShine_AutoCastStart(button.auto.AutoCastShine);
				else
					button.auto.AutoCastShine:Hide();
					AutoCastShine_AutoCastStop(button.auto.AutoCastShine);
				end
			end
		end
	end
end

hooksecurefunc(ChannelFrame.ChannelList, "Update", function(self, ...) 
	AllCheck();
end);

hooksecurefunc(ChannelFrame.ChannelList, "AddChannelButtonInternal", function(self, ...) 
	local button, channelID, clubId, streamInfo = ...;
	if button:ChannelSupportsVoice() and button:GetChannelType() == 4 and not button.helped then
		button.helped = true;
		local auto = CreateFrame("Button", "VoiceChannelAuto"..channelID, button, "PetActionButtonTemplate");
		auto:ClearAllPoints();
		auto:SetPoint("CENTER", button.Speaker, "Center");
		auto:SetScale(0.8);		
		auto.AutoCastable:Show();
		button.auto = auto;

		if VoiceHelper["auto4"] and VoiceHelper["auto4"].clubId == clubId and VoiceHelper["auto4"].streamId == streamInfo.streamId then
			auto.AutoCastShine:Show();
			AutoCastShine_AutoCastStart(auto.AutoCastShine);
		else
			auto.AutoCastShine:Hide();
			AutoCastShine_AutoCastStop(auto.AutoCastShine);
		end
		button.Speaker.Button:HookScript("OnEnter", function(self)
			GameTooltip:ClearLines();
			GameTooltip:AddLine(C_Club.GetClubInfo(self.clubId).name.." - "..C_Club.GetStreamInfo(self.clubId, self.streamId).name,1 ,1 ,1);
			if self:GetParent():GetParent().voiceActive then
				GameTooltip:AddDoubleLine(GetText("VOICE_CHAT_LEAVE"), GetText("KEY_BUTTON1"), 1, 1, 1);
			else
				GameTooltip:AddDoubleLine(GetText("VOICE_CHAT_JOIN"), GetText("KEY_BUTTON1"), 1, 1, 1);
			end
			GameTooltip:AddDoubleLine(GetText("AUTO_JOIN_VOICE"), GetText("KEY_BUTTON2"), 1, 1, 1);
			GameTooltip:Show();
		end);
		button.Speaker.Button:RegisterForClicks("LeftButtonUp", "RightButtonUp");
		
		button.Speaker.Button:SetScript("OnClick", function(self, btn)
			if btn == "RightButton" then
				if VoiceHelper["auto4"] and VoiceHelper["auto4"].clubId == self.clubId and VoiceHelper["auto4"].streamId == self.streamId then
					VoiceHelper["auto4"] = false;
				else
					VoiceHelper["auto4"] = {}
					VoiceHelper["auto4"].clubId = self.clubId;
					VoiceHelper["auto4"].streamId = self.streamId;
				end
				AllCheck();
			else
				self:ToggleActivateChannel();
			end
		end);
	end
end);